const path = require('path');

module.exports = (api, opts = {}) => {
  const env = process.env.NODE_ENV || 'development';
  const esm = process.env.OUTPUT_ESM
  const isDev = env === 'development';
  const isProd = env === 'production';
  const isTest = env === 'test';
  
  const absoluteRuntimepath = path.dirname(require.resolve('@babel/runtime/package.json'));

  const presets = [
    isTest && [
      require('@babel/preset-env').default,
      {
        targets: {
          node: 'current'
        }
      }
    ],
    (isProd || isDev) && [
      require('@babel/preset-env').default,
      {
        useBuiltIns: 'entry',
        corejs: 3,
        modules: false,
        targets: {
          browsers: isProd && esm ? 
          [
            'Chrome >= 60',
            'Safari >= 10.1',
            'iOS >= 10.3',
            'Firefox >= 54',
            'Edge >= 15',
          ] :
          [
            '> 1%',
            'last 2 versions',
            'Firefox ESR',
          ]
        },
        exclude: ['transform-typeof-symbol']
      }
    ],
    require('@vue/babel-preset-jsx')
  ].filter(Boolean);

  const plugins = [
    require('@babel/plugin-syntax-jsx'),
    require('babel-plugin-macros'),
    [
      require('@babel/plugin-transform-destructuring'),
      {
        loose: false
      }
    ],
    [
      require('@babel/plugin-proposal-class-properties').default,
      {
        loose: true,
      },
    ],
    [
      require('@babel/plugin-proposal-object-rest-spread').default,
      {
        useBuiltIns: true,
      },
    ],
    [
      require('@babel/plugin-proposal-decorators').default,
      {
        legacy: true
      },
    ],
    isDev && [
      require('@babel/plugin-transform-runtime').default,
      {
        corejs: false,
        helpers: true,
        regenerator: true,
        useESModules: isDev,
        absoluteRuntime: absoluteRuntimepath,
      },
    ],
    isProd && [
      require('@babel/plugin-transform-regenerator')
    ],
    require('@babel/plugin-syntax-dynamic-import').default,
    isTest && require('babel-plugin-dynamic-import-node')
  ].filter(Boolean);

  return {
    presets,
    plugins
  };
};